@@ -10,18 +10,50 @@ import re
 
 from mercurial import ui, util, error, scmutil, phases
 
-from tortoisehg.util import hglib, shlib, wconfig, bugtraq
+from tortoisehg.util import hglib, shlib, wconfig
 
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.messageentry import MessageEntry
 from tortoisehg.hgqt import qtlib, qscilib, status, cmdui, branchop, revpanel
-from tortoisehg.hgqt import hgrcutil, mq, lfprompt, i18n
-from tortoisehg.util.hgversion import hgversion
+from tortoisehg.hgqt import hgrcutil, mqutil, lfprompt, i18n
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4.Qsci import QsciAPIs
 
+if os.name == 'nt':
+    from tortoisehg.util import bugtraq
+    _hasbugtraq = True
+else:
+    _hasbugtraq = False
+
+def readrepoopts(repo):
+    opts = {}
+    opts['ciexclude'] = repo.ui.config('tortoisehg', 'ciexclude', '')
+    opts['pushafter'] = repo.ui.config('tortoisehg', 'cipushafter', '')
+    opts['autoinc'] = repo.ui.config('tortoisehg', 'autoinc', '')
+    opts['recurseinsubrepos'] = repo.ui.config('tortoisehg', 'recurseinsubrepos', None)
+    opts['bugtraqplugin'] = repo.ui.config('tortoisehg', 'issue.bugtraqplugin', None)
+    opts['bugtraqparameters'] = repo.ui.config('tortoisehg', 'issue.bugtraqparameters', None)
+    if opts['bugtraqparameters']:
+        opts['bugtraqparameters'] = os.path.expandvars(opts['bugtraqparameters'])
+    opts['bugtraqtrigger'] = repo.ui.config('tortoisehg', 'issue.bugtraqtrigger', None)
+
+    return opts
+
+def commitopts2str(opts, mode='commit'):
+    optslist = []
+    for opt, value in opts.iteritems():
+        if opt in ['user', 'date', 'pushafter', 'autoinc',
+                   'recurseinsubrepos']:
+            if mode == 'merge' and opt == 'autoinc':
+                # autoinc does not apply to merge commits
+                continue
+            if value is True:
+                optslist.append('--' + opt)
+            elif value:
+                optslist.append('--%s=%s' % (opt, value))
+    return ' '.join(optslist)
 
 # Technical Debt for CommitWidget
 #  disable commit button while no message is entered or no files are selected
@@ -55,16 +87,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.currentAction = None
         self.currentProgress = None
 
-        opts['ciexclude'] = repo.ui.config('tortoisehg', 'ciexclude', '')
-        opts['pushafter'] = repo.ui.config('tortoisehg', 'cipushafter', '')
-        opts['autoinc'] = repo.ui.config('tortoisehg', 'autoinc', '')
-        opts['recurseinsubrepos'] = repo.ui.config('tortoisehg', 'recurseinsubrepos', None)
-        opts['bugtraqplugin'] = repo.ui.config('tortoisehg', 'issue.bugtraqplugin', None)
-        opts['bugtraqparameters'] = repo.ui.config('tortoisehg', 'issue.bugtraqparameters', None)
-        if opts['bugtraqparameters']:
-            opts['bugtraqparameters'] = os.path.expandvars(opts['bugtraqparameters'])
-        opts['bugtraqtrigger'] = repo.ui.config('tortoisehg', 'issue.bugtraqtrigger', None)
-        self.opts = opts # user, date
+        self.opts = opts = readrepoopts(repo) # user, date
 
         self.stwidget = status.StatusWidget(repo, pats, opts, self)
         self.stwidget.showMessage.connect(self.showMessage)
@@ -117,7 +140,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         tbar.addAction(_('Options')).triggered.connect(self.details)
         tbar.setIconSize(QSize(16,16))
 
-        if self.opts['bugtraqplugin'] != None:
+        if _hasbugtraq and self.opts['bugtraqplugin'] != None:
             # We create the "Show Issues" button, but we delay its setup
             # because creating the bugtraq object is slow and blocks the GUI,
             # which would result in a noticeable slow down while creating the commit widget
@@ -161,7 +184,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             pnhbox = QHBoxLayout()
             self.pnlabel = QLabel()
             pnhbox.addWidget(self.pnlabel)
-            self.pnedit = mq.getPatchNameLineEdit()
+            self.pnedit = mqutil.getPatchNameLineEdit()
             self.pnedit.setMaximumWidth(250)
             pnhbox.addWidget(self.pnedit)
             pnhbox.addStretch()
@@ -253,17 +276,14 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     def commitSetupButton(self):
         ispatch = lambda r: 'qtip' in r.changectx('.').tags()
         notpatch = lambda r: 'qtip' not in r.changectx('.').tags()
-        canamend = lambda r: False
-        # hg >= 2.2 has amend capabilities
-        if hgversion >= '2.2':
-            def canamend(r):
-                if ispatch(r):
-                    return False
-                ctx = r.changectx('.')
-                return not ctx.children() \
-                    and ctx.phase() != phases.public \
-                    and len(ctx.parents()) < 2 \
-                    and len(r.changectx(None).parents()) < 2
+        def canamend(r):
+            if ispatch(r):
+                return False
+            ctx = r.changectx('.')
+            return not ctx.children() \
+                and ctx.phase() != phases.public \
+                and len(ctx.parents()) < 2 \
+                and len(r.changectx(None).parents()) < 2
 
         acts = [
             ('commit', _('Commit changes'), _('Commit'), notpatch),
@@ -348,7 +368,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.pnlabel.setVisible(True)
             self.pnedit.setVisible(True)
             self.pnedit.setFocus()
-            self.pnedit.setText(mq.defaultNewPatchName(self.repo))
+            self.pnedit.setText(mqutil.defaultNewPatchName(self.repo))
             self.pnedit.selectAll()
             self.stwidget.setPatchContext(None)
             refreshwctx = refresh and oldpctx is not None
@@ -438,23 +458,30 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             return self.commit(amend=True)
 
         # Check if we need to change branch first
-        commandlines = []
+        wholecmdlines = []  # [[cmd1, ...], [cmd2, ...], ...]
         if self.branchop:
-            commandlines, newbranch = self.getBranchCommandLine(self.branchop,
-                                                                self.repo)
-            if commandlines is None:
+            cmdlines, newbranch = self.getBranchCommandLine(self.branchop,
+                                                            self.repo)
+            if cmdlines is None:
                 return
+            wholecmdlines.extend(cmdlines)
+
         olist = ('user', 'date')
-        cmdlines = commandlines + mq.mqNewRefreshCommand(self.repo,
-                                          curraction._name == 'qnew',
-                                          self.stwidget, self.pnedit,
-                                          self.msgte.text(), self.opts, olist)
+        cmdlines = mqutil.mqNewRefreshCommand(self.repo,
+                                              curraction._name == 'qnew',
+                                              self.stwidget, self.pnedit,
+                                              self.msgte.text(), self.opts,
+                                              olist)
+        if not cmdlines:
+            return
+        wholecmdlines.extend(cmdlines)
+
         self.repo.incrementBusyCount()
         self.currentAction = curraction._name
         self.currentProgress = _('MQ Action', 'start progress')
         self.progress.emit(*cmdui.startProgress(self.currentProgress, ''))
         self.commitButtonEnable.emit(False)
-        self.runner.run(*cmdlines)
+        self.runner.run(*wholecmdlines)
 
     @pyqtSlot(QString, QString)
     def fileDisplayed(self, wfile, contents):
@@ -494,7 +521,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.msgte.lexer().setAPIs(self._apis)
 
     def bugTrackerPostCommit(self):
-        if self.opts['bugtraqtrigger'] != 'commit':
+        if not _hasbugtraq or self.opts['bugtraqtrigger'] != 'commit':
             return
         # commit already happened, get last message in history
         message = self.lastmessage
@@ -516,7 +543,10 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.setMessage(newMessage)
 
     def details(self):
-        dlg = DetailsDialog(self.opts, self.userhist, self)
+        mode = 'commit'
+        if len(self.repo.parents()) > 1:
+            mode = 'merge'
+        dlg = DetailsDialog(self.opts, self.userhist, self, mode=mode)
         dlg.finished.connect(dlg.deleteLater)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
@@ -566,18 +596,10 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.branchbutton.setText(title)
 
         # Update options label, showing only whitelisted options.
-        opts = []
-        for opt, value in self.opts.iteritems():
-            if opt in ['user', 'date', 'pushafter', 'autoinc',
-                       'recurseinsubrepos']:
-                if value is True:
-                    opts.append('--' + opt)
-                elif value:
-                    opts.append('--%s=%s' % (opt, value))
-
+        opts = commitopts2str(self.opts)
         self.optionslabelfmt = _('<b>Selected Options:</b> %s')
         self.optionslabel.setText(self.optionslabelfmt
-                                  % hglib.tounicode(' '.join(opts)))
+                                  % hglib.tounicode(opts))
         self.optionslabel.setVisible(bool(opts))
 
         # Update parent csinfo widget
@@ -642,7 +664,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.currentProgress = _('Rollback', 'start progress')
         self.progress.emit(*cmdui.startProgress(self.currentProgress, ''))
         self.commitButtonEnable.emit(False)
-        self.runner.run(['rollback'])
+        self.runner.run(['rollback', '--repository', self.repo.root])
         self.stopAction.setEnabled(True)
 
     def updateRecentMessages(self):
@@ -712,18 +734,18 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def saveSettings(self, s, prefix):
         'Save history, etc, in QSettings instance'
-        repoid = str(self.repo[0])
-        lpref = prefix + '/commit/'
-        gpref = 'commit/'
-        s.setValue(lpref+'split', self.split.saveState())
-        self.msgte.saveSettings(s, lpref+'msgte')
-        self.stwidget.saveSettings(s, lpref+'status')
-        s.setValue(gpref+'history-'+repoid, self.msghistory)
-        s.setValue(gpref+'userhist', self.userhist)
-        msg = self.getMessage(True)
         try:
+            repoid = str(self.repo[0])
+            lpref = prefix + '/commit/'
+            gpref = 'commit/'
+            s.setValue(lpref+'split', self.split.saveState())
+            self.msgte.saveSettings(s, lpref+'msgte')
+            self.stwidget.saveSettings(s, lpref+'status')
+            s.setValue(gpref+'history-'+repoid, self.msghistory)
+            s.setValue(gpref+'userhist', self.userhist)
+            msg = self.getMessage(True)
             self.repo.opener('cur-message.txt', 'w').write(msg)
-        except EnvironmentError:
+        except (EnvironmentError, IOError):
             pass
 
     def addMessageToHistory(self, umsg):
@@ -743,6 +765,21 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def commit(self, amend=False):
         repo = self.repo
+        defaultusername = ui.ui().config('ui', 'username')
+        if not defaultusername:
+            res = qtlib.CustomPrompt(
+                _('Default username is not configured'),
+                _('A default username is not configured. '
+                  'This username is used when you commit '
+                  'unless you set a different username on a given repository.\n\n'
+                  'You must configure a default username before being able to commit.\n\n'
+                  'Do you want to configure your default username now?'), self,
+                (_('&Configure'), _('Cancel')), 0, 1, []).run()
+            if res == 0:
+                from tortoisehg.hgqt import settings
+                settings.SettingsDialog(parent=self, focus='ui.username').exec_()
+            return
+  
         try:
             msg = self.getMessage(False)
         except UnicodeEncodeError:
@@ -794,15 +831,19 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                                                                 self.repo)
             if commandlines is None:
                 return
-        self.files = self.stwidget.getChecked('MAR?!S')
-        if not (self.files or brcmd or newbranch):
+        if len(repo.parents()) > 1:
+            merge = True
+            self.files = []
+        else:
+            merge = False
+            self.files = self.stwidget.getChecked('MAR?!S')
+        canemptycommit = bool(brcmd or newbranch or amend)
+        if not (self.files or canemptycommit or merge):
             qtlib.WarningMsgBox(_('No files checked'),
                                 _('No modified files checkmarked for commit'),
                                 parent=self)
             self.stwidget.tv.setFocus()
             return
-        if len(repo.parents()) > 1:
-            self.files = []
 
         user = qtlib.getCurrentUsername(self, self.repo, self.opts)
         if not user:
@@ -879,6 +920,10 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         if amend:
             cmdline.append('--amend')
 
+        if not self.files and canemptycommit and not merge:
+            # make sure to commit empty changeset by excluding all files
+            cmdline.extend(['--exclude', repo.root])
+
         cmdline.append('--')
         cmdline.extend([repo.wjoin(f) for f in self.files])
         if len(repo.parents()) == 1:
@@ -890,6 +935,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
         if self.opts.get('pushafter'):
             cmd = ['push', '--repository', repo.root, self.opts['pushafter']]
+            if newbranch:
+                cmd.append('--new-branch')
             commandlines.append(cmd)
 
         repo.incrementBusyCount()
@@ -931,7 +978,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
 class DetailsDialog(QDialog):
     'Utility dialog for configuring uncommon settings'
-    def __init__(self, opts, userhistory, parent):
+    def __init__(self, opts, userhistory, parent, mode='commit'):
         QDialog.__init__(self, parent)
         self.setWindowTitle(_('%s - commit options') % parent.repo.displayname)
         self.repo = parent.repo
@@ -1050,7 +1097,9 @@ class DetailsDialog(QDialog):
         hbox.addWidget(self.autoinccb)
         hbox.addWidget(self.autoincle)
         hbox.addWidget(autoincsave)
-        layout.addLayout(hbox)
+        if mode != 'merge':
+            #self.autoinccb.setVisible(False)
+            layout.addLayout(hbox)
         
         hbox = QHBoxLayout()
         recursesave = QPushButton(_('Save in Repo'))
@@ -1280,7 +1329,7 @@ class CommitDialog(QDialog):
         self.commit.reload()
         self.updateUndo()
         self.commit.msgte.setFocus()
-        QShortcut(QKeySequence.Refresh, self, self.refresh)
+        qtlib.newshortcutsforstdkey(QKeySequence.Refresh, self, self.refresh)
 
     def done(self, ret):
         self.commit.repo.configChanged.disconnect(self.commit.configChanged)
