@@ -129,10 +129,9 @@ class ManifestModel(QAbstractItemModel):
                   'svn': 'thg-svn-subrepo',
                 }
                 stype = self.subrepoType(index)
-                if stype:
+                if stype in _subrepoType2IcoMap:
                     ic = qtlib.geticon(_subrepoType2IcoMap[stype])
-            ic = qtlib.getoverlaidicon(ic,
-                icOverlay)  # XXX
+            ic = qtlib.getoverlaidicon(ic, icOverlay)
         return ic
 
     def fileStatus(self, index):
