@@ -7,7 +7,6 @@
 
 import os
 import re
-import time
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -18,64 +17,13 @@ from hgext import mq as mqmod
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, cmdui, rejects, qscilib, thgrepo, status
-from tortoisehg.hgqt import qqueue, qreorder, thgimport, messageentry
+from tortoisehg.hgqt import qqueue, qreorder, thgimport, messageentry, mqutil
 from tortoisehg.hgqt.qtlib import geticon
 
 # TODO
 # keep original file name in file list item
 # more wctx functions
 
-def defaultNewPatchName(repo):
-    t = time.strftime('%Y-%m-%d_%H-%M-%S')
-    return t + '_r%d+.diff' % repo['.'].rev()
-
-def getPatchNameLineEdit():
-    patchNameLE = QLineEdit()
-    if hasattr(patchNameLE, 'setPlaceholderText'): # Qt >= 4.7
-        patchNameLE.setPlaceholderText(_('### patch name ###'))
-    return patchNameLE
-
-def getUserOptions(opts, *optionlist):
-    out = []
-    for opt in optionlist:
-        if opt not in opts:
-            continue
-        val = opts[opt]
-        if val is False:
-            continue
-        elif val is True:
-            out.append('--' + opt)
-        else:
-            out.append('--' + opt)
-            out.append(val)
-    return out
-
-def mqNewRefreshCommand(repo, isnew, stwidget, pnwidget, message, opts, olist):
-    if isnew:
-        name = hglib.fromunicode(pnwidget.text())
-        if not name:
-            qtlib.ErrorMsgBox(_('Patch Name Required'),
-                              _('You must enter a patch name'))
-            pnwidget.setFocus()
-            return
-        cmdline = ['qnew', '--repository', repo.root, name]
-    else:
-        cmdline = ['qrefresh', '--repository', repo.root]
-    if message:
-        cmdline += ['--message=' + hglib.fromunicode(message)]
-    cmdline += getUserOptions(opts, *olist)
-    files = ['--'] + [repo.wjoin(x) for x in stwidget.getChecked()]
-    addrem = [repo.wjoin(x) for x in stwidget.getChecked('!?')]
-    if len(files) > 1:
-        cmdline += files
-    else:
-        cmdline += ['--exclude', repo.root]
-    if addrem:
-        cmdlines = [ ['addremove', '-R', repo.root] + addrem, cmdline]
-    else:
-        cmdlines = [cmdline]
-    return cmdlines
-
 class MQPatchesWidget(QDockWidget):
     showMessage = pyqtSignal(unicode)
     output = pyqtSignal(QString, QString)
@@ -531,6 +479,7 @@ class MQPatchesWidget(QDockWidget):
         self.setGuardsAct.setEnabled(False)
         self.qpopAct.setEnabled(bool(applied))
         self.qpopAllAct.setEnabled(bool(applied))
+        self.qreorderAct.setEnabled(bool(repo.thgmqunappliedpatches))
 
     def refreshSelectedGuards(self):
         total = len(self.allguards)
@@ -647,7 +596,7 @@ class MQWidget(QWidget, qtlib.TaskWidget):
         layout.addLayout(mtbarhbox, 0)
         mtbarhbox.setContentsMargins(0, 0, 0, 0)
         self.newCheckBox = QCheckBox(_('New Patch'))
-        self.patchNameLE = getPatchNameLineEdit()
+        self.patchNameLE = mqutil.getPatchNameLineEdit()
         mtbarhbox.addWidget(self.newCheckBox)
         mtbarhbox.addWidget(self.patchNameLE, 1)
 
@@ -705,7 +654,8 @@ class MQWidget(QWidget, qtlib.TaskWidget):
             self.layout().addWidget(self.statusbar)
             self.progress.connect(self.statusbar.progress)
             self.showMessage.connect(self.statusbar.showMessage)
-            QShortcut(QKeySequence.Refresh, self, self.reload)
+            qtlib.newshortcutsforstdkey(QKeySequence.Refresh, self,
+                                        self.reload)
             self.resize(850, 550)
 
         self.loadConfigs()
@@ -717,7 +667,7 @@ class MQWidget(QWidget, qtlib.TaskWidget):
         super(MQWidget, self).closeEvent(event)
 
     def getUserOptions(self, *optionlist):
-        return getUserOptions(self.opts, *optionlist)
+        return mqutil.getUserOptions(self.opts, *optionlist)
 
     @pyqtSlot()
     def onConfigChanged(self):
@@ -843,10 +793,10 @@ class MQWidget(QWidget, qtlib.TaskWidget):
         if self.newCheckBox.isChecked():
             self.finishfunc = lambda ret: self.newCheckBox.setChecked(False)
         optionlist = ('user', 'currentuser', 'git', 'date', 'currentdate')
-        cmdlines = mqNewRefreshCommand(self.repo, self.newCheckBox.isChecked(),
-                                       self.stwidget, self.patchNameLE,
-                                       self.messageEditor.text(), self.opts,
-                                       optionlist)
+        cmdlines = mqutil.mqNewRefreshCommand(self.repo, self.newCheckBox.isChecked(),
+                                              self.stwidget, self.patchNameLE,
+                                              self.messageEditor.text(), self.opts,
+                                              optionlist)
         self.repo.incrementBusyCount()
         self.cmd.run(*cmdlines)
 
@@ -977,7 +927,7 @@ class MQWidget(QWidget, qtlib.TaskWidget):
             self.messageEditor.setEnabled(True)
             self.patchNameLE.setEnabled(True)
             self.patchNameLE.setFocus()
-            self.patchNameLE.setText(defaultNewPatchName(self.repo))
+            self.patchNameLE.setText(mqutil.defaultNewPatchName(self.repo))
             self.patchNameLE.selectAll()
             self.setMessage('')
         else:
