@@ -142,7 +142,6 @@ class RepoTreeItem(object):
 
 
 class RepoItem(RepoTreeItem):
-    shortnameChanged = pyqtSignal()
     def __init__(self, root=None, parent=None):
         RepoTreeItem.__init__(self, parent)
         self._root = root or ''
@@ -183,6 +182,7 @@ class RepoItem(RepoTreeItem):
         self._basenode = basenode
 
     def setShortName(self, uname):
+        uname = unicode(uname)
         if uname != self._shortname:
             self._shortname = uname
 
@@ -276,6 +276,11 @@ class RepoItem(RepoTreeItem):
             try:
                 sri = None
                 if repo is None:
+                    if not os.path.exists(self._root):
+                        self._valid = False
+                        return [self._root]
+                    elif not os.path.exists(os.path.join(self._root, '.hgsub')):
+                        return []  # skip repo creation, which is expensive
                     repo = hg.repository(ui.ui(), self._root)
                 wctx = repo['.']
                 sortkey = lambda x: os.path.basename(util.normpath(repo.wjoin(x)))
@@ -356,7 +361,7 @@ class RepoItem(RepoTreeItem):
                     _('An error occurred while updating the repository hgrc '
                       'file (%s)') % hglib.tounicode(abshgrcpath))
                 return False
-            self.setShortName(shortname)
+            self.setShortName(value.toString())
             return True
         return False
 
