@@ -9,11 +9,7 @@
 import binascii
 import os
 import shlex, subprocess, functools # used by runCustomCommand
-from mercurial import revset, error, patch, commands
-
-# hg >= 2.1
-if hasattr(commands, 'phase'):
-    from mercurial import phases
+from mercurial import revset, error, patch, phases
 
 from tortoisehg.util import hglib, shlib, paths
 from tortoisehg.hgqt.i18n import _
@@ -25,7 +21,7 @@ from tortoisehg.hgqt import cmdui, update, tag, backout, merge, visdiff
 from tortoisehg.hgqt import archive, thgimport, thgstrip, run, purge, bookmark
 from tortoisehg.hgqt import bisect, rebase, resolve, thgrepo, compress, mq
 from tortoisehg.hgqt import qdelete, qreorder, qfold, qrename, shelve
-from tortoisehg.hgqt import matching
+from tortoisehg.hgqt import matching, graft
 
 from tortoisehg.hgqt.repofilter import RepoFilterBar
 from tortoisehg.hgqt.repoview import HgRepoView
@@ -39,8 +35,6 @@ from tortoisehg.hgqt.pbranch import PatchBranchWidget
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-import functools
-
 class RepoWidget(QWidget):
 
     showMessageSignal = pyqtSignal(QString)
@@ -76,7 +70,6 @@ class RepoWidget(QWidget):
         repo.repositoryDestroyed.connect(self.repositoryDestroyed)
         repo.configChanged.connect(self.configChanged)
         self.revsetfilter = False
-        self.ubranch = u''
         self.bundle = None  # bundle file name [local encoding]
         self.bundlesource = None  # source URL of incoming bundle [unicode]
         self.outgoingMode = False
@@ -175,7 +168,7 @@ class RepoWidget(QWidget):
         self.filterbar.progress.connect(self.progress)
         self.filterbar.showMessage.connect(self.showMessage)
         self.filterbar.setRevisionSet.connect(self.setRevisionSet)
-        self.filterbar.clearRevisionSet.connect(self.clearRevisionSet)
+        self.filterbar.clearRevisionSet.connect(self._unapplyRevisionSet)
         self.filterbar.filterToggled.connect(self.filterToggled)
         self.filterbar.hide()
         self.revsetfilter = self.filterbar.filtercb.isChecked()
@@ -277,8 +270,8 @@ class RepoWidget(QWidget):
         """Returns the expected title for this widget [unicode]"""
         if self.bundle:
             return _('%s <incoming>') % self.repo.shortname
-        elif self.ubranch:
-            return u'%s [%s]' % (self.repo.shortname, self.ubranch)
+        elif self.repomodel.branch():
+            return u'%s [%s]' % (self.repo.shortname, self.repomodel.branch())
         else:
             return self.repo.shortname
 
@@ -386,6 +379,7 @@ class RepoWidget(QWidget):
         w.linkActivated.connect(self._openLink)
         w.showMessage.connect(self.showMessage)
         w.grepRequested.connect(self.grep)
+        w.revsetFilterRequested.connect(self.setFilter)
         return w
 
     def createSyncWidget(self):
@@ -422,7 +416,7 @@ class RepoWidget(QWidget):
         else:
             self.showIcon.emit(QIcon())
 
-    @pyqtSlot(QString, QString)
+    @pyqtSlot(QString)
     def setFilter(self, filter):
         self.filterbar.revsetle.setText(filter)
         self.filterbar.setVisible(True)
@@ -514,8 +508,13 @@ class RepoWidget(QWidget):
     @pyqtSlot()
     def clearRevisionSet(self):
         self.filterbar.revsetle.clear()
+        return self._unapplyRevisionSet()
+
+    @pyqtSlot()
+    def _unapplyRevisionSet(self):
         self.toolbarVisibilityChanged.emit()
         self.outgoingMode = False
+        self.repoview.enablefilterpalette(False)
         if not self.revset:
             return False
         self.revset = []
@@ -538,6 +537,7 @@ class RepoWidget(QWidget):
             self.refresh()
         self.repoview.resetBrowseHistory(self.revset)
         self._reload_rev = self.revset[0]
+        self.repoview._paletteswitcher.enablefilterpalette(revs)
 
     @pyqtSlot(bool)
     def filterToggled(self, checked):
@@ -550,22 +550,32 @@ class RepoWidget(QWidget):
     def setOutgoingNodes(self, nodes):
         self.filterbar.revsetle.setText('outgoing()')
         self.setRevisionSet([self.repo[n].rev() for n in nodes])
+        self.outgoingMode = True
 
         w = self.setInfoBar(qtlib.ConfirmInfoBar,
                             _('%d outgoing changesets') % len(nodes))
         assert w
 
         # Read the tortoisehg.defaultpush setting to determine what to push
-        # by default
+        # by default, and set the button label and action accordingly
         acceptbuttontext = _('Push')
         defaultpush = self.repo.ui.config('tortoisehg', 'defaultpush', 'all')
+        rev = None
+        branch = None
+        pushall = False
         if defaultpush == 'branch':
-            acceptbuttontext = _('Push current branch')
+            branch = self.repo['.'].branch()
+            acceptbuttontext = _('Push current branch (%s)') \
+                % hglib.tounicode(branch)
         elif defaultpush == 'revision':
-            acceptbuttontext = _('Push current revision')
+            rev = self.repo['.'].rev()
+            acceptbuttontext = _('Push current revision (%d)') % rev
+        else:
+            pushall = True
 
         w.acceptButton.setText(acceptbuttontext)
-        w.accepted.connect(lambda: self.push(False))  # TODO: to the same URL
+        w.accepted.connect(lambda: self.push(False,
+            rev=rev, branch=branch, pushall=pushall))  # TODO: to the same URL
         w.rejected.connect(self.clearRevisionSet)
 
     def createGrepWidget(self):
@@ -714,8 +724,6 @@ class RepoWidget(QWidget):
             if os.path.exists(self.repo.sjoin('undo')):
                 try:
                     args = self.repo.opener('undo.desc', 'r').read().splitlines()
-                    if args[1] != 'commit':
-                        return None
                     return args[1], int(args[0])
                 except (IOError, IndexError, ValueError):
                     pass
@@ -773,12 +781,15 @@ class RepoWidget(QWidget):
         # Filter revision set in case revisions were removed
         self.revset = [r for r in self.revset if r < len(self.repo)]
         self.repomodel = HgRepoListModel(self.repo, self.repoview.colselect[0],
-                                         self.ubranch, self.revset,
+                                         self.filterbar.branch(), self.revset,
                                          self.revsetfilter, self)
         self.repomodel.filled.connect(self.modelFilled)
         self.repomodel.loaded.connect(self.modelLoaded)
         self.repomodel.showMessage.connect(self.showMessage)
+        oldmodel = self.repoview.model()
         self.repoview.setModel(self.repomodel)
+        if oldmodel:
+            oldmodel.deleteLater()
         try:
             self._last_series = self.repo.mq.series[:]
         except AttributeError:
@@ -810,8 +821,10 @@ class RepoWidget(QWidget):
         if not cw.canswitch():
             return
         ctx = self.repo.changectx(rev)
-        if rev is None or ('mq' in self.repo.extensions() and 'qtip' in ctx.tags()):
-            # Clicking on working copy switches to commit tab
+        if rev is None or ('mq' in self.repo.extensions() and 'qtip' in ctx.tags()
+                           and self.repo['.'].rev() == rev):
+            # Clicking on working copy or on the topmost applied patch
+            # (_if_ it is also the working copy parent) switches to the commit tab
             tw.setCurrentIndex(self.commitTabIndex)
         else:
             # Clicking on a normal revision switches from commit tab
@@ -987,9 +1000,8 @@ class RepoWidget(QWidget):
             self.taskTabsWidget.tabBar().hide()
 
     @pyqtSlot(QString, bool)
-    def setBranch(self, branch, allparents=True):
+    def setBranch(self, branch, allparents):
         'Change the branch filter'
-        self.ubranch = branch
         self.repomodel.setBranch(branch=branch, allparents=allparents)
         self.titleChanged.emit(self.title())
         if self.revset:
@@ -1064,14 +1076,13 @@ class RepoWidget(QWidget):
         self.syncDemand.get().pull()
     def outgoing(self):
         self.syncDemand.get().outgoing()
-        self.outgoingMode = True
-    def push(self, confirm=True):
+    def push(self, confirm=True, **kwargs):
         """Call sync push.
 
         If confirm is False, the user will not be prompted for
         confirmation. If confirm is True, the prompt might be used.
         """
-        self.syncDemand.get().push(confirm)
+        self.syncDemand.get().push(confirm, **kwargs)
         self.outgoingMode = False
 
     @pyqtSlot()
@@ -1093,6 +1104,7 @@ class RepoWidget(QWidget):
         if len(selection) == 0:
             return
 
+        self.menuselection = selection
         if self.bundle:
             if len(selection) == 1:
                 self.bundlemenu.exec_(point)
@@ -1102,7 +1114,6 @@ class RepoWidget(QWidget):
                 self.outgoingcmenu.exec_(point)
                 return
 
-        self.menuselection = selection
         allunapp = False
         if 'mq' in self.repo.extensions():
             for rev in selection:
@@ -1213,10 +1224,12 @@ class RepoWidget(QWidget):
         entry(menu, None, isrev, _('Update...'), 'hg-update',
               self.updateToRevision)
         entry(menu)
-        entry(menu, None, isctx, _('Visual diff...'), 'visualdiff',
+        entry(menu, None, isctx, _('Diff to parent...'), 'visualdiff',
               self.visualDiffRevision)
         entry(menu, None, isrev, _('Diff to local...'), 'ldiff',
               self.visualDiffToLocal)
+        entry(menu, None, isctx, _('Browse at rev...'), 'hg-annotate',
+              self.manifestRevision)
         entry(menu, None, isrev, _('Similar revisions...'), 'view-filter',
               self.matchRevision)
         entry(menu)
@@ -1249,16 +1262,14 @@ class RepoWidget(QWidget):
               self.copyPatch)
         entry(menu)
 
-        # hg >= 2.1
-        if hasattr(commands, 'phase'):
-            submenu = menu.addMenu(_('Change phase to'))
-            for pnum, pname in enumerate(phases.phasenames):
-                entry(submenu, None, isrev, pname, None,
-                      functools.partial(self.changePhase, pnum))
-            entry(menu)
+        submenu = menu.addMenu(_('Change phase to'))
+        for pnum, pname in enumerate(phases.phasenames):
+            entry(submenu, None, isrev, pname, None,
+                  functools.partial(self.changePhase, pnum))
+        entry(menu)
 
-        entry(menu, 'transplant', fixed, _('Transplant to local'), 'hg-transplant',
-              self.transplantRevisions)
+        entry(menu, None, fixed, _('Graft to local...'), 'hg-transplant',
+              self.graftRevisions)
 
         if 'mq' in exs or 'rebase' in exs:
             submenu = menu.addMenu(_('Modify history'))
@@ -1284,7 +1295,8 @@ class RepoWidget(QWidget):
               self.rupdate)
 
         def _setupCustomSubmenu(menu):
-            tools, toolnames = hglib.tortoisehgtools(self.repo.ui, 'repowidget')
+            tools, toollist = hglib.tortoisehgtools(self.repo.ui,
+                selectedlocation='workbench.revdetails.custom-menu')
             if not tools:
                 return
 
@@ -1296,10 +1308,12 @@ class RepoWidget(QWidget):
 
             entry(menu)
             submenu = menu.addMenu(_('Custom Tools'))
-            for name in toolnames:
-                info = tools[name]
-                location = info.get('location', '').replace(' ', '').split(',')
-                if 'repowidget' not in location:
+            for name in toollist:
+                if name == '|':
+                    entry(submenu)
+                    continue
+                info = tools.get(name, None)
+                if info is None:
                     continue
                 command = info.get('command', None)
                 if not command:
@@ -1325,6 +1339,13 @@ class RepoWidget(QWidget):
             self.singlecmenu = menu
             self.singlecmenuitems = items
 
+    def _gotoAncestor(self):
+        ancestor = self.repo[self.menuselection[0]]
+        for rev in self.menuselection[1:]:
+            ctx = self.repo[rev]
+            ancestor = ancestor.ancestor(ctx)
+        self.goto(ancestor.rev())
+
     def generatePairMenu(self):
         def dagrange():
             revA, revB = self.menuselection
@@ -1402,10 +1423,6 @@ class RepoWidget(QWidget):
             dlg = compress.CompressDialog(self.repo, revs, self)
             dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
-        def gotoAncestor():
-            ctxa = self.repo[self.menuselection[0]]
-            ctxb = self.repo[self.menuselection[1]]
-            self.goto(ctxa.ancestor(ctxb).rev())
 
         menu = QMenu(self)
         for name, cb, icon in (
@@ -1423,8 +1440,10 @@ class RepoWidget(QWidget):
                 (_('Bisect - Bad, Good...'), bisectReverse, 'hg-bisect-bad-good'),
                 (_('Compress History...'), compressDlg, 'hg-compress'),
                 (None, None, None),
-                (_('Goto common ancestor'), gotoAncestor, 'hg-merge'),
-                (_('Similar revisions...'), self.matchRevision, 'view-filter')
+                (_('Goto common ancestor'), self._gotoAncestor, 'hg-merge'),
+                (_('Similar revisions...'), self.matchRevision, 'view-filter'),
+                (None, None, None),
+                (_('Graft Selected to local...'), self.graftRevisions, 'hg-transplant'),
                 ):
             if name is None:
                 menu.addSeparator()
@@ -1435,13 +1454,6 @@ class RepoWidget(QWidget):
             a.triggered.connect(cb)
             menu.addAction(a)
 
-        if 'transplant' in self.repo.extensions():
-            menu.addSeparator()
-            a = QAction(_('Transplant Selected to local'), self)
-            a.setIcon(qtlib.getmenuicon('hg-transplant'))
-            a.triggered.connect(self.transplantRevisions)
-            menu.addAction(a)
-
         if 'reviewboard' in self.repo.extensions():
             menu.addSeparator()
             a = QAction(_('Post Selected to Review Board...'), self)
@@ -1509,7 +1521,10 @@ class RepoWidget(QWidget):
                 (_('Export Selected...'), exportSel, 'hg-export'),
                 (_('Email Selected...'), emailSel, 'mail-forward'),
                 (None, None, None),
+                (_('Goto common ancestor'), self._gotoAncestor, 'hg-merge'),
                 (_('Similar revisions...'), self.matchRevision, 'view-filter'),
+                (None, None, None),
+                (_('Graft Selected to local...'), self.graftRevisions, 'hg-transplant'),
                 ):
             if name is None:
                 menu.addSeparator()
@@ -1520,13 +1535,6 @@ class RepoWidget(QWidget):
             a.triggered.connect(cb)
             menu.addAction(a)
 
-        if 'transplant' in self.repo.extensions():
-            menu.addSeparator()
-            a = QAction(_('Transplant Selected to local'), self)
-            a.setIcon(qtlib.getmenuicon('hg-transplant'))
-            a.triggered.connect(self.transplantRevisions)
-            menu.addAction(a)
-
         if 'reviewboard' in self.repo.extensions():
             a = QAction(_('Post Selected to Review Board...'), self)
             a.triggered.connect(self.sendToReviewBoard)
@@ -1681,28 +1689,15 @@ class RepoWidget(QWidget):
         dlg.exec_()
 
     def matchRevision(self):
-        hasmatching = True
-        try:
-            # hg >= 2.2
-            from mercurial.revset import matching as matchingkeyword
-        except:
-            hasmatching = False
-        if hasmatching:
-            revlist = self.rev
-            if len(self.menuselection) > 1:
-                revlist = '|'.join([str(rev) for rev in self.menuselection])
-            dlg = matching.MatchDialog(self.repo, revlist, self)
-            if dlg.exec_():
-                self.setFilter(dlg.revsetexpression)
-        else:
-            # We cannot find similar revisions
-            # without the matching revset keyword
-            qtlib.WarningMsgBox(_('Incorrect Mercurial version'),
-                _('In order to use the "Find Similar revisions" '
-                'functionality, you must use a mercurial version above 2.1.'))
+        revlist = self.rev
+        if len(self.menuselection) > 1:
+            revlist = '|'.join([str(rev) for rev in self.menuselection])
+        dlg = matching.MatchDialog(self.repo, revlist, self)
+        if dlg.exec_():
+            self.setFilter(dlg.revsetexpression)
 
     def pushAll(self):
-        self.syncDemand.forward('push', True)
+        self.syncDemand.forward('push', False, pushall=True)
 
     def pushToRevision(self):
         # Do not ask for confirmation
@@ -1713,6 +1708,9 @@ class RepoWidget(QWidget):
         self.syncDemand.forward('push', False,
             branch=self.repo[self.rev].branch())
 
+    def manifestRevision(self):
+        run.manifest(self.repo.ui, repo=self.repo, rev=self.rev)
+
     def mergeWithRevision(self):
         dlg = merge.MergeDialog(self.rev, self.repo, self)
         dlg.exec_()
@@ -1734,11 +1732,17 @@ class RepoWidget(QWidget):
         dlg.finished.connect(dlg.deleteLater)
         dlg.exec_()
 
-    def transplantRevisions(self):
-        cmdline = ['transplant', '--repository', self.repo.root]
-        for rev in self.repoview.selectedRevisions():
-            cmdline.append(str(rev))
-        self.runCommand(cmdline)
+    def graftRevisions(self):
+        """Graft selected revision on top of working directory parent"""
+        revlist = []
+        for rev in sorted(self.repoview.selectedRevisions()):
+            revlist.append(str(rev))
+        if not revlist:
+            revlist = [self.rev]
+        dlg = graft.GraftDialog(self.repo, self, source=revlist)
+        dlg.finished.connect(dlg.deleteLater)
+        if dlg.valid:
+            dlg.exec_()
 
     def backoutToRevision(self):
         dlg = backout.BackoutDialog(self.rev, self.repo, self)
@@ -1814,6 +1818,7 @@ class RepoWidget(QWidget):
                 class Writable(object):
                     def write(self, *args, **opts): _ui.write(*args, **opts)
                     def close(self): pass
+                    def __len__(self): return 0
                 commands.export(_ui, self.repo, self.rev, output=Writable())
             else:
                 revs = self.rev and self.menuselection or None
@@ -1831,7 +1836,10 @@ class RepoWidget(QWidget):
         if returnval:
             return output
         else:
-            QApplication.clipboard().setText(hglib.tounicode(output))
+            mdata = QMimeData()
+            mdata.setData('text/x-diff', output)  # for lossless import
+            mdata.setText(hglib.tounicode(output))
+            QApplication.clipboard().setMimeData(mdata)
 
     def copyHash(self):
         clip = QApplication.clipboard()
@@ -2024,6 +2032,7 @@ class RepoWidget(QWidget):
         # 2. Expand internal workbench variables
         vars = {
             'ROOT': self.repo.root,
+            'REVID': str(self.repo[self.rev]),
             'REV': self.rev
         }
         for var in vars:
@@ -2042,7 +2051,16 @@ class RepoWidget(QWidget):
             return self.runCommand(cmd)
 
         # Otherwise, run the selected command in the brackground
-        return subprocess.Popen(command, cwd=self.repo.root)
+        try:
+            res = subprocess.Popen(command, cwd=self.repo.root)
+        except OSError, ex:
+            res = 1
+            qtlib.ErrorMsgBox(_('Failed to execute custom command'),
+                _('The command "%s" could not be executed.') % hglib.tounicode(command),
+                _('The following error message was returned:\n\n"%s"\n\n'
+                'Please check that the command path is valid and '
+                'that it is a valid application') % hglib.tounicode(ex.strerror))
+        return res
 
     def runCommand(self, *cmdlines):
         if self.runner.core.running():
